#ifndef TD3DDrawPrimitiveStridedDataH
#define TD3DDrawPrimitiveStridedDataH

// ==========================================================================
// File:         TD3DDrawPrimitiveStridedData.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons)
// Description:  This file defines the TD3DDrawPrimitiveStridedData Component 
//
// "TDx_Draw3D_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Draw3D_Library_Defns.H"
#include "TDx_Draw3D_Library_Functions.H"
// --------------------------------------------------------------------------
// external classes used by TD3DDrawPrimitiveStridedData properties.
#include "TD3DDP_PtrStride.H"

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TD3DDrawPrimitiveStridedData
// Description: The TD3DDrawPrimitiveStridedData component wraps multiple 
//              DirectX D3DDRAWPRIMITIVESTRIDEDDATA structures describing 
//              flexible vertex buffer formats. 
//              
//              This component is used by the 
//              TDx_3DDevice::DrawIndexedPrimitiveStrided(), 
//              TDx_3DDevice::DrawPrimitiveStrided() and 
//              TDx_3DVertexBuffer::ProcessVerticesStrided() methods to hold 
//              the pointers and memory strides of the vertex components of a 
//              primitive. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TD3DDrawPrimitiveStridedData : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TD3DDrawPrimitiveStridedData : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TD3DDrawPrimitiveStridedData component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    ArraySize
    // Description: The ArraySize property contains the number of internal 
    //              D3DDRAWPRIMITIVESTRIDEDDATA structures wrapped inside the 
    //              TD3DDrawPrimitiveStridedData component. 
    //              By default the ArraySize is initialized to 1, meaning 
    //              that this TD3DDrawPrimitiveStridedData component wraps a 
    //              single D3DDRAWPRIMITIVESTRIDEDDATA structure. To wrap 
    //              multiple structures, set the 
    //              TD3DDrawPrimitiveStridedData::ArraySize to the total 
    //              number of internal structures desired. 
    // ----------------------------------------------------------------------
    __property dword ArraySize = { read=FGetArraySize, write=FSetArraySize, 
    default=1 }; 

    // ----------------------------------------------------------------------
    // Property:    Diffuse
    // Description: The Diffuse property contains a TD3DDP_PtrStride 
    //              component with the Diffuse color information for a 
    //              collection of vertices, stored as 8-8-8-8 RGBA values. 
    // ----------------------------------------------------------------------
    __property TD3DDP_PtrStride* Diffuse[dword pArrayIndex] = { 
    read=FGetDiffuse, write=FSetDiffuse }; 

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TD3DDrawPrimitiveStridedData 
    //              method or fget/fset. eg. D3D_OK or DDERR_SURFACELOST or 
    //              TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=D3D_OK }; 

    // ----------------------------------------------------------------------
    // Property:    Normal
    // Description: The Normal property contains a TD3DDP_PtrStride component 
    //              with an array of the Normal vectors for a collection of 
    //              vertices, each vector being a three element array of 
    //              float values. 
    // ----------------------------------------------------------------------
    __property TD3DDP_PtrStride* Normal[dword pArrayIndex] = { 
    read=FGetNormal, write=FSetNormal }; 

    // ----------------------------------------------------------------------
    // Property:    Position
    // Description: The Position property contains a TD3DDP_PtrStride 
    //              component with an array of the position vectors for a 
    //              collection of vertices, each vector being a three element 
    //              array of float values. 
    // ----------------------------------------------------------------------
    __property TD3DDP_PtrStride* Position[dword pArrayIndex] = { 
    read=FGetPosition, write=FSetPosition }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property contains the size in bytes of a single 
    //              internal D3DDRAWPRIMITIVESTRIDEDDATA structure. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    SizeAll
    // Description: The SizeAll property contains the size in bytes of all 
    //              internal D3DDRAWPRIMITIVESTRIDEDDATA structures. This 
    //              result is the same as multiplying 
    //              TD3DDrawPrimitiveStridedData::ArraySize and 
    //              TD3DDrawPrimitiveStridedData::Size. 
    // ----------------------------------------------------------------------
    __property dword SizeAll = { read=FGetSizeAll, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    Specular
    // Description: The Specluar property contains a TD3DDP_PtrStride 
    //              component with the Specular color information for a 
    //              collection of vertices, stored as 8-8-8-8 RGBA values. 
    // ----------------------------------------------------------------------
    __property TD3DDP_PtrStride* Specular[dword pArrayIndex] = { 
    read=FGetSpecular, write=FSetSpecular }; 

    // ----------------------------------------------------------------------
    // Property:    TextureCoords
    // Description: The TextureCoords property contains an eight element 
    //              array of TD3DDP_PtrStride components, with each element 
    //              being a texture coordinate array for the vertices. 
    //              
    //              Which of the eight elements used for a given texture 
    //              stage is determined by calling 
    //              TDx_3DDevice::SetTextureStageState() with 
    //              D3DTSS_TEXCOORDINDEX set. 
    // ----------------------------------------------------------------------
    __property TD3DDP_PtrStride* TextureCoords[dword pArrayIndex][dword 
    pIndex] = { read=FGetTextureCoords, write=FSetTextureCoords }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TD3DDrawPrimitiveStridedData's internal 
    //              D3DDRAWPRIMITIVESTRIDEDDATA structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // Params:      pArrayIndex - 
    //                The ArrayIndex parameter determines which internal 
    //                structure needs to be cleared. Correct values range from 
    //                0 to TD3DDrawPrimitiveStridedData::ArraySize-1. 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear( dword pArrayIndex ); 

    // ----------------------------------------------------------------------
    // Method:      ClearAll()
    // Description: The ClearAll() method can be used to clear the contents 
    //              of all the internal D3DDRAWPRIMITIVESTRIDEDDATA 
    //              structures of the TD3DDrawPrimitiveStridedData component. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall ClearAll(); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TD3DDrawPrimitiveStridedData(TComponent* Owner); 
    virtual __fastcall ~TD3DDrawPrimitiveStridedData(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property D3DDRAWPRIMITIVESTRIDEDDATA* 
    Internal_D3DDRAWPRIMITIVESTRIDEDDATA_Ptr = { 
    read=FGetInternal_D3DDRAWPRIMITIVESTRIDEDDATA_Ptr, nodefault }; 

    void __fastcall Internal_D3DDRAWPRIMITIVESTRIDEDDATA_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    dword __fastcall FGetArraySize(); 
    void __fastcall FSetArraySize( dword pArraySize ); 
    TD3DDP_PtrStride* __fastcall FGetDiffuse( dword pArrayIndex ); 
    void __fastcall FSetDiffuse( dword pArrayIndex, TD3DDP_PtrStride* 
    pDiffuse ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    TD3DDP_PtrStride* __fastcall FGetNormal( dword pArrayIndex ); 
    void __fastcall FSetNormal( dword pArrayIndex, TD3DDP_PtrStride* pNormal 
    ); 
    TD3DDP_PtrStride* __fastcall FGetPosition( dword pArrayIndex ); 
    void __fastcall FSetPosition( dword pArrayIndex, TD3DDP_PtrStride* 
    pPosition ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 
    dword __fastcall FGetSizeAll(); 
    TD3DDP_PtrStride* __fastcall FGetSpecular( dword pArrayIndex ); 
    void __fastcall FSetSpecular( dword pArrayIndex, TD3DDP_PtrStride* 
    pSpecular ); 
    TD3DDP_PtrStride* __fastcall FGetTextureCoords( dword pArrayIndex, dword 
    pIndex ); 
    void __fastcall FSetTextureCoords( dword pArrayIndex, dword pIndex, 
    TD3DDP_PtrStride* pTextureCoords ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    D3DDRAWPRIMITIVESTRIDEDDATA* __fastcall 
    FGetInternal_D3DDRAWPRIMITIVESTRIDEDDATA_Ptr(); 
    void __fastcall FSetInternal_D3DDRAWPRIMITIVESTRIDEDDATA( dword 
    pArrayIndex, D3DDRAWPRIMITIVESTRIDEDDATA* pD3DDRAWPRIMITIVESTRIDEDDATA ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    dword fArraySize; 
    TD3DDP_PtrStride* fDiffuse; 
    HRESULT fErrorValue; 
    TD3DDP_PtrStride* fNormal; 
    TD3DDP_PtrStride* fPosition; 
    dword fSize; 
    dword fSizeAll; 
    TD3DDP_PtrStride* fSpecular; 
    TD3DDP_PtrStride* fTextureCoords; 

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    D3DDRAWPRIMITIVESTRIDEDDATA* fD3DDRAWPRIMITIVESTRIDEDDATA; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
