// --------------------------------------------------------------------------

// ==========================================================================
// File:         TD3DLVertex.CPP
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons)
// Description:  This file contains the code for the TD3DLVertex Component 
//
// "TDx_Draw3D_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
// #includes ...
// --------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop
// --------------------------------------------------------------------------
#include "TD3DLVertex.H"
// --------------------------------------------------------------------------
#pragma link "TDx_Library_Defns"
#pragma link "TDx_Library_Functions"
#pragma link "TDx_Draw3D_Library_Defns"
#pragma link "TDx_Draw3D_Library_Functions"

// --------------------------------------------------------------------------
// Object Registration...
// --------------------------------------------------------------------------
#if (__BORLANDC__ >= 0x0530) // BCB Version 3+
    #pragma package(smart_init)
#endif
// --------------------------------------------------------------------------
static inline void ValidCtrCheck(TD3DLVertex*)
{
    new TD3DLVertex(NULL);
}
// --------------------------------------------------------------------------
namespace Td3dlvertex
{
    #if (__BORLANDC__ >= 0x0530) // BCB Version 3+
    void __fastcall PACKAGE Register()
    #else
    void __fastcall Register()
    #endif
    {
        TComponentClass classes[1] = {__classid(TD3DLVertex)};
        RegisterComponents("TDx_Draw3D", classes, 0);
    }
}

// --------------------------------------------------------------------------
// Constructor: TD3DLVertex::TD3DLVertex()
// Description: The default constructor for the TD3DLVertex object.
// --------------------------------------------------------------------------
__fastcall TD3DLVertex::TD3DLVertex(TComponent* Owner)
    : TComponent(Owner)
{
fD3DLVERTEX=NULL;
fArraySize=0;
ArraySize=1;
ClearAll();
}
// --------------------------------------------------------------------------
// Destructor:  TD3DLVertex::~TD3DLVertex()
// Description: The destructor for the TD3DLVertex object.
// --------------------------------------------------------------------------
__fastcall TD3DLVertex::~TD3DLVertex()
{
// destroy internals
if (ArraySize!=0) ArraySize = 0;
}
// --------------------------------------------------------------------------
// Property:    ArraySize
// Description: The ArraySize property contains the number of internal 
//              D3DLVERTEX structures wrapped inside the TD3DLVertex 
//              component. 
//              By default the ArraySize is initialized to 1, meaning that 
//              this TD3DLVertex component wraps a single D3DLVERTEX 
//              structure. To wrap multiple structures, set the 
//              TD3DLVertex::ArraySize to the total number of internal 
//              structures desired. 
// --------------------------------------------------------------------------
dword __fastcall TD3DLVertex::FGetArraySize()
{
return fArraySize;
}
// --------------------------------------------------------------------------
void __fastcall TD3DLVertex::FSetArraySize( dword pArraySize )
{
// check for size==0
if (pArraySize==0)
    {
    // de-allocate old memory
    delete[] fD3DLVERTEX;
    fD3DLVERTEX = NULL;
    fArraySize = 0;
    return;
    }

// is anything allocated?
if (fArraySize==0)
    {
    // nope. need to allocate memory for pointers to internal structures
    fD3DLVERTEX = new D3DLVERTEX[pArraySize];
    if (fD3DLVERTEX==NULL)
        {
        fArraySize = 0;
        fErrorValue = TDX_ERROR;
        if (FOnError) FOnError(this, "TD3DLVERTEX::FSetArraySize()", "TDX_ERROR", "Could not allocate memory for storing internal D3DLVERTEX structure pointers.");
        return;
        }
    // setup defaults
    fArraySize = pArraySize;
    ClearAll();
    }
else
    // already allocated... has the size increased?
    if (pArraySize>fArraySize)
        {
        // allocate space for the new structures
        D3DLVERTEX* temp_D3DLVERTEX = new D3DLVERTEX[pArraySize];
        if (temp_D3DLVERTEX==NULL)
            {
            fErrorValue = TDX_ERROR;
            if (FOnError) FOnError(this, "TD3DLVERTEX::FSetArraySize()", "TDX_ERROR", "Could not allocate extra memory for storing internal D3DLVERTEX structures.");
            return;
            }
        // clear new data areas
        ZeroMemory( &temp_D3DLVERTEX[fArraySize], sizeof(D3DLVERTEX)*(pArraySize-fArraySize) );
        // duplicate existing data
        CopyMemory( &temp_D3DLVERTEX[0], &fD3DLVERTEX[0], sizeof(D3DLVERTEX)*fArraySize);
        // de-allocate old memory
        delete[] fD3DLVERTEX;
        // re-assign the new memory
        fD3DLVERTEX = temp_D3DLVERTEX;
        // store new array size
        fArraySize = pArraySize;
        }
    else
        // already allocated... has the size reduced?
        if (pArraySize<fArraySize)
            {
            // allocate space for the new structures
            D3DLVERTEX* temp_D3DLVERTEX = new D3DLVERTEX[pArraySize];
            if (temp_D3DLVERTEX==NULL)
                {
                fErrorValue = TDX_ERROR;
                if (FOnError) FOnError(this, "TD3DLVERTEX::FSetArraySize()", "TDX_ERROR", "Could not allocate memory for storing internal D3DLVERTEX structures.");
                return;
                }
            // duplicate existing data
            CopyMemory( &temp_D3DLVERTEX[0], &fD3DLVERTEX[0], sizeof(D3DLVERTEX)*pArraySize);
            // de-allocate the old memory
            delete[] fD3DLVERTEX;
            // re-assign the new memory
            fD3DLVERTEX = temp_D3DLVERTEX;
            // store new array size
            fArraySize = pArraySize;
            }
// else - size is the same.
}
// --------------------------------------------------------------------------
// Property:    Color
// Description: The Color property contains a D3DCOLOR defining the color 
//              component of the vertex. 
//              
//              This property is unioned with the color property of this 
//              object. 
// --------------------------------------------------------------------------
D3DCOLOR __fastcall TD3DLVertex::FGetColor( dword pArrayIndex )
{
// array is allocated?
if (fArraySize==0)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FGetColor()", "TDX_ERROR", "Internal Array size is 0.");
    return (D3DCOLOR) NULL;
    }
// range check supplied index
if (pArrayIndex>=fArraySize)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FGetColor()", "TDX_ERROR", "Supplied index ["+IntToStr(pArrayIndex)+"] is out of bounds [0.."+IntToStr(fArraySize-1)+"]");
    return (D3DCOLOR) NULL;
    }

return fD3DLVERTEX[pArrayIndex].dcColor;
}
// --------------------------------------------------------------------------
void __fastcall TD3DLVertex::FSetColor( dword pArrayIndex, D3DCOLOR pColor )
{
// array is allocated?
if (fArraySize==0)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FSetColor()", "TDX_ERROR", "Internal Array size is 0.");
    return;
    }
// range check supplied index
if (pArrayIndex>=fArraySize)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FSetColor()", "TDX_ERROR", "Supplied index ["+IntToStr(pArrayIndex)+"] is out of bounds [0.."+IntToStr(fArraySize-1)+"]");
    return;
    }

fD3DLVERTEX[pArrayIndex].dcColor = pColor;
}
// --------------------------------------------------------------------------
// Property:    ErrorValue
// Description: The ErrorValue property contains the last error value 
//              returned from a call to a TD3DLVertex method or fget/fset. 
//              eg. D3D_OK or DDERR_SURFACELOST or TDX_ERROR 
// --------------------------------------------------------------------------
HRESULT __fastcall TD3DLVertex::FGetErrorValue()
{
return fErrorValue;
}
// --------------------------------------------------------------------------
void __fastcall TD3DLVertex::FSetErrorValue( HRESULT pErrorValue )
{
fErrorValue = pErrorValue;
}
// --------------------------------------------------------------------------
// Property:    Size
// Description: The Size property contains the size in bytes of a single 
//              internal D3DLVERTEX structure. 
// --------------------------------------------------------------------------
dword __fastcall TD3DLVertex::FGetSize()
{
return fSize;
}
// --------------------------------------------------------------------------
void __fastcall TD3DLVertex::FSetSize( dword pSize )
{
fSize = pSize;
}
// --------------------------------------------------------------------------
// Property:    SizeAll
// Description: The SizeAll property contains the size in bytes of all 
//              internal D3DLVERTEX structures. This result is the same as 
//              multiplying TD3DLVertex::ArraySize and TD3DLVertex::Size. 
// --------------------------------------------------------------------------
dword __fastcall TD3DLVertex::FGetSizeAll()
{
return fArraySize * fSize;
}
// --------------------------------------------------------------------------
// Property:    Specular
// Description: The Specular property contains a D3DCOLOR defining the 
//              specular component of the vertex. 
//              
//              This property is unioned with the specular property of this 
//              object. 
// --------------------------------------------------------------------------
D3DCOLOR __fastcall TD3DLVertex::FGetSpecular( dword pArrayIndex )
{
// array is allocated?
if (fArraySize==0)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FGetSpecular()", "TDX_ERROR", "Internal Array size is 0.");
    return (D3DCOLOR) NULL;
    }
// range check supplied index
if (pArrayIndex>=fArraySize)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FGetSpecular()", "TDX_ERROR", "Supplied index ["+IntToStr(pArrayIndex)+"] is out of bounds [0.."+IntToStr(fArraySize-1)+"]");
    return (D3DCOLOR) NULL;
    }

return fD3DLVERTEX[pArrayIndex].dcSpecular;
}
// --------------------------------------------------------------------------
void __fastcall TD3DLVertex::FSetSpecular( dword pArrayIndex, D3DCOLOR pSpecular )
{
// array is allocated?
if (fArraySize==0)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FSetSpecular()", "TDX_ERROR", "Internal Array size is 0.");
    return;
    }
// range check supplied index
if (pArrayIndex>=fArraySize)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FSetSpecular()", "TDX_ERROR", "Supplied index ["+IntToStr(pArrayIndex)+"] is out of bounds [0.."+IntToStr(fArraySize-1)+"]");
    return;
    }

fD3DLVERTEX[pArrayIndex].dcSpecular = pSpecular;
}
// --------------------------------------------------------------------------
// Property:    TU
// Description: The TU property contains a D3DVALUE specifying the U texture 
//              coordinate of the vertex. 
//              
//              This property is unioned with the tu property of this object. 
// --------------------------------------------------------------------------
D3DVALUE __fastcall TD3DLVertex::FGetTU( dword pArrayIndex )
{
// array is allocated?
if (fArraySize==0)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FGetTU()", "TDX_ERROR", "Internal Array size is 0.");
    return (D3DVALUE) NULL;
    }
// range check supplied index
if (pArrayIndex>=fArraySize)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FGetTU()", "TDX_ERROR", "Supplied index ["+IntToStr(pArrayIndex)+"] is out of bounds [0.."+IntToStr(fArraySize-1)+"]");
    return (D3DVALUE) NULL;
    }

return fD3DLVERTEX[pArrayIndex].dvTU;
}
// --------------------------------------------------------------------------
void __fastcall TD3DLVertex::FSetTU( dword pArrayIndex, D3DVALUE pTU )
{
// array is allocated?
if (fArraySize==0)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FSetTU()", "TDX_ERROR", "Internal Array size is 0.");
    return;
    }
// range check supplied index
if (pArrayIndex>=fArraySize)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FSetTU()", "TDX_ERROR", "Supplied index ["+IntToStr(pArrayIndex)+"] is out of bounds [0.."+IntToStr(fArraySize-1)+"]");
    return;
    }

fD3DLVERTEX[pArrayIndex].dvTU = pTU;
}
// --------------------------------------------------------------------------
// Property:    TV
// Description: The TV property contains a D3DVALUE specifying the V texture 
//              coordinate of the vertex. 
//              
//              This property is unioned with the tv property of this object. 
// --------------------------------------------------------------------------
D3DVALUE __fastcall TD3DLVertex::FGetTV( dword pArrayIndex )
{
// array is allocated?
if (fArraySize==0)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FGetTV()", "TDX_ERROR", "Internal Array size is 0.");
    return (D3DVALUE) NULL;
    }
// range check supplied index
if (pArrayIndex>=fArraySize)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FGetTV()", "TDX_ERROR", "Supplied index ["+IntToStr(pArrayIndex)+"] is out of bounds [0.."+IntToStr(fArraySize-1)+"]");
    return (D3DVALUE) NULL;
    }

return fD3DLVERTEX[pArrayIndex].dvTV;
}
// --------------------------------------------------------------------------
void __fastcall TD3DLVertex::FSetTV( dword pArrayIndex, D3DVALUE pTV )
{
// array is allocated?
if (fArraySize==0)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FSetTV()", "TDX_ERROR", "Internal Array size is 0.");
    return;
    }
// range check supplied index
if (pArrayIndex>=fArraySize)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FSetTV()", "TDX_ERROR", "Supplied index ["+IntToStr(pArrayIndex)+"] is out of bounds [0.."+IntToStr(fArraySize-1)+"]");
    return;
    }

fD3DLVERTEX[pArrayIndex].dvTV = pTV;
}
// --------------------------------------------------------------------------
// Property:    X
// Description: The X property contains a D3DVALUE specifying the X model 
//              coordinate of the vertex. 
//              
//              This property is unioned with the x property of this object. 
//              
//              The vertex position is defined relative to the origin 
//              specified for the objects model space. 
// --------------------------------------------------------------------------
D3DVALUE __fastcall TD3DLVertex::FGetX( dword pArrayIndex )
{
// array is allocated?
if (fArraySize==0)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FGetX()", "TDX_ERROR", "Internal Array size is 0.");
    return (D3DVALUE) NULL;
    }
// range check supplied index
if (pArrayIndex>=fArraySize)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FGetX()", "TDX_ERROR", "Supplied index ["+IntToStr(pArrayIndex)+"] is out of bounds [0.."+IntToStr(fArraySize-1)+"]");
    return (D3DVALUE) NULL;
    }

return fD3DLVERTEX[pArrayIndex].dvX;
}
// --------------------------------------------------------------------------
void __fastcall TD3DLVertex::FSetX( dword pArrayIndex, D3DVALUE pX )
{
// array is allocated?
if (fArraySize==0)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FSetX()", "TDX_ERROR", "Internal Array size is 0.");
    return;
    }
// range check supplied index
if (pArrayIndex>=fArraySize)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FSetX()", "TDX_ERROR", "Supplied index ["+IntToStr(pArrayIndex)+"] is out of bounds [0.."+IntToStr(fArraySize-1)+"]");
    return;
    }

fD3DLVERTEX[pArrayIndex].dvX = pX;
}
// --------------------------------------------------------------------------
// Property:    Y
// Description: The Y property contains a D3DVALUE specifying the X model 
//              coordinate of the vertex. 
//              
//              This property is unioned with the y property of this object. 
//              
//              The vertex position is defined relative to the origin 
//              specified for the objects model space. 
// --------------------------------------------------------------------------
D3DVALUE __fastcall TD3DLVertex::FGetY( dword pArrayIndex )
{
// array is allocated?
if (fArraySize==0)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FGetY()", "TDX_ERROR", "Internal Array size is 0.");
    return (D3DVALUE) NULL;
    }
// range check supplied index
if (pArrayIndex>=fArraySize)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FGetY()", "TDX_ERROR", "Supplied index ["+IntToStr(pArrayIndex)+"] is out of bounds [0.."+IntToStr(fArraySize-1)+"]");
    return (D3DVALUE) NULL;
    }

return fD3DLVERTEX[pArrayIndex].dvY;
}
// --------------------------------------------------------------------------
void __fastcall TD3DLVertex::FSetY( dword pArrayIndex, D3DVALUE pY )
{
// array is allocated?
if (fArraySize==0)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FSetY()", "TDX_ERROR", "Internal Array size is 0.");
    return;
    }
// range check supplied index
if (pArrayIndex>=fArraySize)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FSetY()", "TDX_ERROR", "Supplied index ["+IntToStr(pArrayIndex)+"] is out of bounds [0.."+IntToStr(fArraySize-1)+"]");
    return;
    }

fD3DLVERTEX[pArrayIndex].dvY = pY;
}
// --------------------------------------------------------------------------
// Property:    Z
// Description: The Z property contains a D3DVALUE specifying the X model 
//              coordinate of the vertex. 
//              
//              This property is unioned with the z property of this object. 
//              
//              The vertex position is defined relative to the origin 
//              specified for the objects model space. 
// --------------------------------------------------------------------------
D3DVALUE __fastcall TD3DLVertex::FGetZ( dword pArrayIndex )
{
// array is allocated?
if (fArraySize==0)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FGetZ()", "TDX_ERROR", "Internal Array size is 0.");
    return (D3DVALUE) NULL;
    }
// range check supplied index
if (pArrayIndex>=fArraySize)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FGetZ()", "TDX_ERROR", "Supplied index ["+IntToStr(pArrayIndex)+"] is out of bounds [0.."+IntToStr(fArraySize-1)+"]");
    return (D3DVALUE) NULL;
    }

return fD3DLVERTEX[pArrayIndex].dvZ;
}
// --------------------------------------------------------------------------
void __fastcall TD3DLVertex::FSetZ( dword pArrayIndex, D3DVALUE pZ )
{
// array is allocated?
if (fArraySize==0)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FSetZ()", "TDX_ERROR", "Internal Array size is 0.");
    return;
    }
// range check supplied index
if (pArrayIndex>=fArraySize)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FSetZ()", "TDX_ERROR", "Supplied index ["+IntToStr(pArrayIndex)+"] is out of bounds [0.."+IntToStr(fArraySize-1)+"]");
    return;
    }

fD3DLVERTEX[pArrayIndex].dvZ = pZ;
}
// --------------------------------------------------------------------------
// Property:    color
// Description: The color property contains a D3DCOLOR defining the color 
//              component of the vertex. 
//              
//              This property is unioned with the Color property of this 
//              object. 
// --------------------------------------------------------------------------
D3DCOLOR __fastcall TD3DLVertex::FGetcolor( dword pArrayIndex )
{
// array is allocated?
if (fArraySize==0)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FGetcolor()", "TDX_ERROR", "Internal Array size is 0.");
    return (D3DCOLOR) NULL;
    }
// range check supplied index
if (pArrayIndex>=fArraySize)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FGetcolor()", "TDX_ERROR", "Supplied index ["+IntToStr(pArrayIndex)+"] is out of bounds [0.."+IntToStr(fArraySize-1)+"]");
    return (D3DCOLOR) NULL;
    }

return fD3DLVERTEX[pArrayIndex].color;
}
// --------------------------------------------------------------------------
void __fastcall TD3DLVertex::FSetcolor( dword pArrayIndex, D3DCOLOR pcolor )
{
// array is allocated?
if (fArraySize==0)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FSetcolor()", "TDX_ERROR", "Internal Array size is 0.");
    return;
    }
// range check supplied index
if (pArrayIndex>=fArraySize)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FSetcolor()", "TDX_ERROR", "Supplied index ["+IntToStr(pArrayIndex)+"] is out of bounds [0.."+IntToStr(fArraySize-1)+"]");
    return;
    }

fD3DLVERTEX[pArrayIndex].color = pcolor;
}
// --------------------------------------------------------------------------
// Property:    specular
// Description: The specular property contains a D3DCOLOR defining the 
//              specular component of the vertex. 
//              
//              This property is unioned with the Specular property of this 
//              object. 
// --------------------------------------------------------------------------
D3DCOLOR __fastcall TD3DLVertex::FGetspecular( dword pArrayIndex )
{
// array is allocated?
if (fArraySize==0)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FGetspecular()", "TDX_ERROR", "Internal Array size is 0.");
    return (D3DCOLOR) NULL;
    }
// range check supplied index
if (pArrayIndex>=fArraySize)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FGetspecular()", "TDX_ERROR", "Supplied index ["+IntToStr(pArrayIndex)+"] is out of bounds [0.."+IntToStr(fArraySize-1)+"]");
    return (D3DCOLOR) NULL;
    }

return fD3DLVERTEX[pArrayIndex].specular;
}
// --------------------------------------------------------------------------
void __fastcall TD3DLVertex::FSetspecular( dword pArrayIndex, D3DCOLOR pspecular )
{
// array is allocated?
if (fArraySize==0)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FSetspecular()", "TDX_ERROR", "Internal Array size is 0.");
    return;
    }
// range check supplied index
if (pArrayIndex>=fArraySize)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FSetspecular()", "TDX_ERROR", "Supplied index ["+IntToStr(pArrayIndex)+"] is out of bounds [0.."+IntToStr(fArraySize-1)+"]");
    return;
    }

fD3DLVERTEX[pArrayIndex].specular = pspecular;
}
// --------------------------------------------------------------------------
// Property:    tu
// Description: The tu property contains a D3DVALUE specifying the U texture 
//              coordinate of the vertex. 
//              
//              This property is unioned with the TU property of this object. 
// --------------------------------------------------------------------------
D3DVALUE __fastcall TD3DLVertex::FGettu( dword pArrayIndex )
{
// array is allocated?
if (fArraySize==0)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FGettu()", "TDX_ERROR", "Internal Array size is 0.");
    return (D3DVALUE) NULL;
    }
// range check supplied index
if (pArrayIndex>=fArraySize)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FGettu()", "TDX_ERROR", "Supplied index ["+IntToStr(pArrayIndex)+"] is out of bounds [0.."+IntToStr(fArraySize-1)+"]");
    return (D3DVALUE) NULL;
    }

return fD3DLVERTEX[pArrayIndex].tu;
}
// --------------------------------------------------------------------------
void __fastcall TD3DLVertex::FSettu( dword pArrayIndex, D3DVALUE ptu )
{
// array is allocated?
if (fArraySize==0)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FSettu()", "TDX_ERROR", "Internal Array size is 0.");
    return;
    }
// range check supplied index
if (pArrayIndex>=fArraySize)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FSettu()", "TDX_ERROR", "Supplied index ["+IntToStr(pArrayIndex)+"] is out of bounds [0.."+IntToStr(fArraySize-1)+"]");
    return;
    }

fD3DLVERTEX[pArrayIndex].tu = ptu;
}
// --------------------------------------------------------------------------
// Property:    tv
// Description: The tv property contains a D3DVALUE specifying the V texture 
//              coordinate of the vertex. 
//              
//              This property is unioned with the TV property of this object. 
// --------------------------------------------------------------------------
D3DVALUE __fastcall TD3DLVertex::FGettv( dword pArrayIndex )
{
// array is allocated?
if (fArraySize==0)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FGettv()", "TDX_ERROR", "Internal Array size is 0.");
    return (D3DVALUE) NULL;
    }
// range check supplied index
if (pArrayIndex>=fArraySize)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FGettv()", "TDX_ERROR", "Supplied index ["+IntToStr(pArrayIndex)+"] is out of bounds [0.."+IntToStr(fArraySize-1)+"]");
    return (D3DVALUE) NULL;
    }

return fD3DLVERTEX[pArrayIndex].tv;
}
// --------------------------------------------------------------------------
void __fastcall TD3DLVertex::FSettv( dword pArrayIndex, D3DVALUE ptv )
{
// array is allocated?
if (fArraySize==0)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FSettv()", "TDX_ERROR", "Internal Array size is 0.");
    return;
    }
// range check supplied index
if (pArrayIndex>=fArraySize)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FSettv()", "TDX_ERROR", "Supplied index ["+IntToStr(pArrayIndex)+"] is out of bounds [0.."+IntToStr(fArraySize-1)+"]");
    return;
    }

fD3DLVERTEX[pArrayIndex].tv = ptv;
}
// --------------------------------------------------------------------------
// Property:    x
// Description: The x property contains a D3DVALUE specifying the X model 
//              coordinate of the vertex. 
//              
//              This property is unioned with the X property of this object. 
//              
//              The vertex position is defined relative to the origin 
//              specified for the objects model space. 
// --------------------------------------------------------------------------
D3DVALUE __fastcall TD3DLVertex::FGetx( dword pArrayIndex )
{
// array is allocated?
if (fArraySize==0)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FGetx()", "TDX_ERROR", "Internal Array size is 0.");
    return (D3DVALUE) NULL;
    }
// range check supplied index
if (pArrayIndex>=fArraySize)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FGetx()", "TDX_ERROR", "Supplied index ["+IntToStr(pArrayIndex)+"] is out of bounds [0.."+IntToStr(fArraySize-1)+"]");
    return (D3DVALUE) NULL;
    }

return fD3DLVERTEX[pArrayIndex].x;
}
// --------------------------------------------------------------------------
void __fastcall TD3DLVertex::FSetx( dword pArrayIndex, D3DVALUE px )
{
// array is allocated?
if (fArraySize==0)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FSetx()", "TDX_ERROR", "Internal Array size is 0.");
    return;
    }
// range check supplied index
if (pArrayIndex>=fArraySize)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FSetx()", "TDX_ERROR", "Supplied index ["+IntToStr(pArrayIndex)+"] is out of bounds [0.."+IntToStr(fArraySize-1)+"]");
    return;
    }

fD3DLVERTEX[pArrayIndex].x = px;
}
// --------------------------------------------------------------------------
// Property:    y
// Description: The y property contains a D3DVALUE specifying the Y model 
//              coordinate of the vertex. 
//              
//              This property is unioned with the Y property of this object. 
//              
//              The vertex position is defined relative to the origin 
//              specified for the objects model space. 
// --------------------------------------------------------------------------
D3DVALUE __fastcall TD3DLVertex::FGety( dword pArrayIndex )
{
// array is allocated?
if (fArraySize==0)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FGety()", "TDX_ERROR", "Internal Array size is 0.");
    return (D3DVALUE) NULL;
    }
// range check supplied index
if (pArrayIndex>=fArraySize)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FGety()", "TDX_ERROR", "Supplied index ["+IntToStr(pArrayIndex)+"] is out of bounds [0.."+IntToStr(fArraySize-1)+"]");
    return (D3DVALUE) NULL;
    }

return fD3DLVERTEX[pArrayIndex].y;
}
// --------------------------------------------------------------------------
void __fastcall TD3DLVertex::FSety( dword pArrayIndex, D3DVALUE py )
{
// array is allocated?
if (fArraySize==0)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FSety()", "TDX_ERROR", "Internal Array size is 0.");
    return;
    }
// range check supplied index
if (pArrayIndex>=fArraySize)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FSety()", "TDX_ERROR", "Supplied index ["+IntToStr(pArrayIndex)+"] is out of bounds [0.."+IntToStr(fArraySize-1)+"]");
    return;
    }

fD3DLVERTEX[pArrayIndex].y = py;
}
// --------------------------------------------------------------------------
// Property:    z
// Description: The z property contains a D3DVALUE specifying the Z model 
//              coordinate of the vertex. 
//              
//              This property is unioned with the Z property of this object. 
//              
//              The vertex position is defined relative to the origin 
//              specified for the objects model space. 
// --------------------------------------------------------------------------
D3DVALUE __fastcall TD3DLVertex::FGetz( dword pArrayIndex )
{
// array is allocated?
if (fArraySize==0)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FGetz()", "TDX_ERROR", "Internal Array size is 0.");
    return (D3DVALUE) NULL;
    }
// range check supplied index
if (pArrayIndex>=fArraySize)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FGetz()", "TDX_ERROR", "Supplied index ["+IntToStr(pArrayIndex)+"] is out of bounds [0.."+IntToStr(fArraySize-1)+"]");
    return (D3DVALUE) NULL;
    }

return fD3DLVERTEX[pArrayIndex].z;
}
// --------------------------------------------------------------------------
void __fastcall TD3DLVertex::FSetz( dword pArrayIndex, D3DVALUE pz )
{
// array is allocated?
if (fArraySize==0)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FSetz()", "TDX_ERROR", "Internal Array size is 0.");
    return;
    }
// range check supplied index
if (pArrayIndex>=fArraySize)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::FSetz()", "TDX_ERROR", "Supplied index ["+IntToStr(pArrayIndex)+"] is out of bounds [0.."+IntToStr(fArraySize-1)+"]");
    return;
    }

fD3DLVERTEX[pArrayIndex].z = pz;
}
// --------------------------------------------------------------------------
// Method:      Clear()
// Description: The Clear() method can be used to clear the contents of the 
//              TD3DLVertex's internal D3DLVERTEX structure. 
//              Note: if you have manually linked a structure member to a 
//              chunk of memory, make sure you release this memory before 
//              calling Clear(). 
// Params:      pArrayIndex - 
//                The ArrayIndex parameter determines which internal structure 
//                needs to be cleared. Correct values range from 0 to 
//                TD3DLVertex::ArraySize-1. 
// --------------------------------------------------------------------------
void __fastcall TD3DLVertex::Clear( dword pArrayIndex )
{
// array is allocated?
if (fArraySize==0)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::Clear()", "TDX_ERROR", "Internal Array size is 0.");
    return;
    }
// range check supplied index
if (pArrayIndex>=fArraySize)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::Clear()", "TDX_ERROR", "Supplied index ["+IntToStr(pArrayIndex)+"] is out of bounds [0.."+IntToStr(fArraySize-1)+"]");
    return;
    }

ZeroMemory( &fD3DLVERTEX[pArrayIndex], sizeof(D3DLVERTEX) );
}


// --------------------------------------------------------------------------
// Method:      ClearAll()
// Description: The ClearAll() method can be used to clear the contents of 
//              all the internal D3DLVERTEX structures of the TD3DLVertex 
//              component. 
//              Note: if you have manually linked a structure member to a 
//              chunk of memory, make sure you release this memory before 
//              calling Clear(). 
// --------------------------------------------------------------------------
void __fastcall TD3DLVertex::ClearAll()
{
// array is allocated?
if (fArraySize==0)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, "TD3DLVertex::Clear()", "TDX_ERROR", "Internal Array size is 0.");
    return;
    }

ZeroMemory( &fD3DLVERTEX[0], sizeof(D3DLVERTEX)*fArraySize );
}


// --------------------------------------------------------------------------
// Internal Structure Access 
// --------------------------------------------------------------------------
D3DLVERTEX* __fastcall TD3DLVertex::FGetInternal_D3DLVERTEX_Ptr()
{
// check for errors
if (fArraySize==0)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError(this, "TD3DLVertex::FGetInternal_D3DLVERTEX_Ptr()", "TDX_ERROR", "Internal Array size is 0." );
    return NULL;
    }
return fD3DLVERTEX;
}
// --------------------------------------------------------------------------
void __fastcall TD3DLVertex::FSetInternal_D3DLVERTEX( dword pArrayIndex, D3DLVERTEX* pD3DLVERTEX )
{
// check for errors
if (fArraySize==0)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError(this, "TD3DLVertex::FSetInternal_D3DLVERTEX()", "TDX_ERROR", "Internal Array size is 0." );
    return;
    }
if (pD3DLVERTEX==NULL)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError(this, "TD3DLVertex::FSetInternal_D3DLVERTEX()", "TDX_ERROR", "The supplied D3DLVERTEX* was NULL" );
    return;
    }
CopyMemory( &fD3DLVERTEX[pArrayIndex], pD3DLVERTEX, sizeof(D3DLVERTEX) );
Internal_D3DLVERTEX_Update();
}
// --------------------------------------------------------------------------
void __fastcall TD3DLVertex::Internal_D3DLVERTEX_Update()
{
}

// --------------------------------------------------------------------------
