#ifndef TD3DLinePatternH
#define TD3DLinePatternH

// ==========================================================================
// File:         TD3DLinePattern.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons)
// Description:  This file defines the TD3DLinePattern Component 
//
// "TDx_Draw3D_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Draw3D_Library_Defns.H"
#include "TDx_Draw3D_Library_Functions.H"

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TD3DLinePattern
// Description: The TD3DLinePattern component wraps the DirectX 
//              D3DLINEPATTERN structure describing a line pattern. 
//              
//              The values contained in this component are used by the 
//              D3DRENDERSTATE_LINEPATTERN render state in the 
//              D3DRENDERSTATETYPE enumerated type. 
//              A line pattern specifies how a line is drawn and the pattern 
//              is always the same regardless of where it is started. 
//              This is different to stippling, which affects rendering and 
//              imitates transparency. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TD3DLinePattern : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TD3DLinePattern : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    LinePattern
    // Description: The LinePattern property contains the bits that specify 
    //              the line pattern. 
    //              
    //              Some simple line patterns examples may be: 
    //              52428: 1100110011001100 to produce a dotted line. 
    //              43690: 1010101010101010 to produce a point line. 
    //              61166: 1110111011101110 to produce a dashed line. 
    // ----------------------------------------------------------------------
    __property word LinePattern = { read=FGetLinePattern, 
    write=FSetLinePattern, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    RepeatFactor
    // Description: The RepeatFactor property contains the number of times to 
    //              duplicate each series of 1s and 0s specified in 
    //              LinePattern. 
    //              
    //              This allows an application to stretch the line pattern if 
    //              neccessary. 
    //              D3DPMISCCAPS_LINEPATTERNREP must be present in 
    //              TD3DPrimCaps::MiscCaps to set values greater than 1. 
    // ----------------------------------------------------------------------
    __property word RepeatFactor = { read=FGetRepeatFactor, 
    write=FSetRepeatFactor, default=0 }; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TD3DLinePattern component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TD3DLinePattern method or 
    //              fget/fset. eg. D3D_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=D3D_OK }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property contains the size in bytes of the 
    //              internal D3DLINEPATTERN structure. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, nodefault }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TD3DLinePattern's internal D3DLINEPATTERN structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear(); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TD3DLinePattern(TComponent* Owner); 
    virtual __fastcall ~TD3DLinePattern(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property D3DLINEPATTERN* Internal_D3DLINEPATTERN_Ptr = { 
    read=FGetInternal_D3DLINEPATTERN_Ptr, nodefault }; 

    void __fastcall Internal_D3DLINEPATTERN_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    word __fastcall FGetLinePattern(); 
    void __fastcall FSetLinePattern( word pLinePattern ); 
    word __fastcall FGetRepeatFactor(); 
    void __fastcall FSetRepeatFactor( word pRepeatFactor ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    D3DLINEPATTERN* __fastcall FGetInternal_D3DLINEPATTERN_Ptr(); 
    void __fastcall FSetInternal_D3DLINEPATTERN( D3DLINEPATTERN* 
    pD3DLINEPATTERN ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    HRESULT fErrorValue; 
    dword fSize; 

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    D3DLINEPATTERN fD3DLINEPATTERN; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
