#ifndef TD3DTLVertexH
#define TD3DTLVertexH

// ==========================================================================
// File:         TD3DTLVertex.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons)
// Description:  This file defines the TD3DTLVertex Component 
//
// "TDx_Draw3D_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Draw3D_Library_Defns.H"
#include "TDx_Draw3D_Library_Functions.H"

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TD3DTLVertex
// Description: The TD3DTLVertex component wraps multiple DirectX D3DTLVERTEX 
//              structures defining transformed and lit vertices in terms of 
//              their screen coordinates and color. 
//              
//              Setting D3DFVF_XYZRHW in the vertex format description will 
//              pass the vertices directly to the rasteriser without 
//              modification. 
//              Transformed and lit vertices are clipped using the X, Y, 
//              Width and Height values of TD3DViewPort and z-coordinates are 
//              clipped to [0,1] 
//              Setting D3DDP_DONOTCLIP when rendering can prevent clipping. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TD3DTLVertex : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TD3DTLVertex : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TD3DTLVertex component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    ArraySize
    // Description: The ArraySize property contains the number of internal 
    //              D3DTLVERTEX structures wrapped inside the TD3DTLVertex 
    //              component. 
    //              By default the ArraySize is initialized to 1, meaning 
    //              that this TD3DTLVertex component wraps a single 
    //              D3DTLVERTEX structure. To wrap multiple structures, set 
    //              the TD3DTLVertex::ArraySize to the total number of 
    //              internal structures desired. 
    // ----------------------------------------------------------------------
    __property dword ArraySize = { read=FGetArraySize, write=FSetArraySize, 
    default=1 }; 

    // ----------------------------------------------------------------------
    // Property:    Color
    // Description: The Color property contains a D3DCOLOR defining the color 
    //              component of the vertex. 
    //              
    //              This property is unioned with the color property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property D3DCOLOR Color[dword pArrayIndex] = { read=FGetColor, 
    write=FSetColor }; 

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TD3DTLVertex method or 
    //              fget/fset. eg. D3D_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=D3D_OK }; 

    // ----------------------------------------------------------------------
    // Property:    RHW
    // Description: The RHW property contains a D3DVALUE defining the 
    //              reciprocal of homogenous w from the homogenous 
    //              coordinates (x,y,z,w). 
    //              
    //              Usually this is 1 divided by the objects z-distance from 
    //              the origin. 
    //              This property is unioned with the rhw property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property D3DVALUE RHW[dword pArrayIndex] = { read=FGetRHW, 
    write=FSetRHW }; 

    // ----------------------------------------------------------------------
    // Property:    SX
    // Description: The SX property contains a D3DVALUE defining the X axis 
    //              screen coordinate of the vertex. 
    //              
    //              This property is unioned with the sx property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property D3DVALUE SX[dword pArrayIndex] = { read=FGetSX, write=FSetSX 
    }; 

    // ----------------------------------------------------------------------
    // Property:    SY
    // Description: The SY property contains a D3DVALUE defining the Y axis 
    //              screen coordinate of the vertex. 
    //              
    //              This property is unioned with the sy property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property D3DVALUE SY[dword pArrayIndex] = { read=FGetSY, write=FSetSY 
    }; 

    // ----------------------------------------------------------------------
    // Property:    SZ
    // Description: The SZ property contains a D3DVALUE defining the Z axis 
    //              screen coordinate of the vertex. 
    //              
    //              This property is unioned with the sz property of this 
    //              component and cannot exceed 0.99999 and not be clipped. 
    // ----------------------------------------------------------------------
    __property D3DVALUE SZ[dword pArrayIndex] = { read=FGetSZ, write=FSetSZ 
    }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property contains the size in bytes of a single 
    //              internal D3DTLVERTEX structure. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    SizeAll
    // Description: The SizeAll property contains the size in bytes of all 
    //              internal D3DTLVERTEX structures. This result is the same 
    //              as multiplying TD3DTLVertex::ArraySize and 
    //              TD3DTLVertex::Size. 
    // ----------------------------------------------------------------------
    __property dword SizeAll = { read=FGetSizeAll, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    Specular
    // Description: The Specular property contains a D3DCOLOR defining the 
    //              specular color component of the vertex. 
    //              
    //              This property is unioned with the specular property of 
    //              this component. 
    // ----------------------------------------------------------------------
    __property D3DCOLOR Specular[dword pArrayIndex] = { read=FGetSpecular, 
    write=FSetSpecular }; 

    // ----------------------------------------------------------------------
    // Property:    TU
    // Description: The TU property contains a D3DVALUE specifying the U 
    //              texture coordinate of the vertex. 
    //              
    //              This property is unioned with the tu property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property D3DVALUE TU[dword pArrayIndex] = { read=FGetTU, write=FSetTU 
    }; 

    // ----------------------------------------------------------------------
    // Property:    TV
    // Description: The TV property contains a D3DVALUE specifying the U 
    //              texture coordinate of the vertex. 
    //              
    //              This property is unioned with the tv property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property D3DVALUE TV[dword pArrayIndex] = { read=FGetTV, write=FSetTV 
    }; 

    // ----------------------------------------------------------------------
    // Property:    color
    // Description: The color property contains a D3DCOLOR defining the color 
    //              component of the vertex. 
    //              
    //              This property is unioned with the Color property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property D3DCOLOR color[dword pArrayIndex] = { read=FGetcolor, 
    write=FSetcolor }; 

    // ----------------------------------------------------------------------
    // Property:    rhw
    // Description: The rhw property contains a D3DVALUE defining the 
    //              reciprocal of homogenous w from the homogenous 
    //              coordinates (x,y,z,w). 
    //              
    //              Usually this is 1 divided by the objects z-distance from 
    //              the origin. 
    //              This property is unioned with the RHW property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property D3DVALUE rhw[dword pArrayIndex] = { read=FGetrhw, 
    write=FSetrhw }; 

    // ----------------------------------------------------------------------
    // Property:    specular
    // Description: The specular property contains a D3DCOLOR defining the 
    //              specular color component of the vertex. 
    //              
    //              This property is unioned with the Specular property of 
    //              this component. 
    // ----------------------------------------------------------------------
    __property D3DCOLOR specular[dword pArrayIndex] = { read=FGetspecular, 
    write=FSetspecular }; 

    // ----------------------------------------------------------------------
    // Property:    sx
    // Description: The sx property contains a D3DVALUE defining the X axis 
    //              screen coordinate of the vertex. 
    //              
    //              This property is unioned with the SX property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property D3DVALUE sx[dword pArrayIndex] = { read=FGetsx, write=FSetsx 
    }; 

    // ----------------------------------------------------------------------
    // Property:    sy
    // Description: The sy property contains a D3DVALUE defining the Y axis 
    //              screen coordinate of the vertex. 
    //              
    //              This property is unioned with the SY property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property D3DVALUE sy[dword pArrayIndex] = { read=FGetsy, write=FSetsy 
    }; 

    // ----------------------------------------------------------------------
    // Property:    sz
    // Description: The sz property contains a D3DVALUE defining the Z axis 
    //              screen coordinate of the vertex. 
    //              
    //              This property is unioned with the SZ property of this 
    //              component and cannot exceed 0.99999 and not be clipped. 
    // ----------------------------------------------------------------------
    __property D3DVALUE sz[dword pArrayIndex] = { read=FGetsz, write=FSetsz 
    }; 

    // ----------------------------------------------------------------------
    // Property:    tu
    // Description: The tu property contains a D3DVALUE specifying the U 
    //              texture coordinate of the vertex. 
    //              
    //              This property is unioned with the TU property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property D3DVALUE tu[dword pArrayIndex] = { read=FGettu, write=FSettu 
    }; 

    // ----------------------------------------------------------------------
    // Property:    tv
    // Description: The tv property contains a D3DVALUE specifying the U 
    //              texture coordinate of the vertex. 
    //              
    //              This property is unioned with the TV property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property D3DVALUE tv[dword pArrayIndex] = { read=FGettv, write=FSettv 
    }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TD3DTLVertex's internal D3DTLVERTEX structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // Params:      pArrayIndex - 
    //                The ArrayIndex parameter determines which internal 
    //                structure needs to be cleared. Correct values range from 
    //                0 to TD3DTLVertex::ArraySize-1. 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear( dword pArrayIndex ); 

    // ----------------------------------------------------------------------
    // Method:      ClearAll()
    // Description: The ClearAll() method can be used to clear the contents 
    //              of all the internal D3DTLVERTEX structures of the 
    //              TD3DTLVertex component. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall ClearAll(); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TD3DTLVertex(TComponent* Owner); 
    virtual __fastcall ~TD3DTLVertex(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property D3DTLVERTEX* Internal_D3DTLVERTEX_Ptr = { 
    read=FGetInternal_D3DTLVERTEX_Ptr, nodefault }; 

    void __fastcall Internal_D3DTLVERTEX_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    dword __fastcall FGetArraySize(); 
    void __fastcall FSetArraySize( dword pArraySize ); 
    D3DCOLOR __fastcall FGetColor( dword pArrayIndex ); 
    void __fastcall FSetColor( dword pArrayIndex, D3DCOLOR pColor ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    D3DVALUE __fastcall FGetRHW( dword pArrayIndex ); 
    void __fastcall FSetRHW( dword pArrayIndex, D3DVALUE pRHW ); 
    D3DVALUE __fastcall FGetSX( dword pArrayIndex ); 
    void __fastcall FSetSX( dword pArrayIndex, D3DVALUE pSX ); 
    D3DVALUE __fastcall FGetSY( dword pArrayIndex ); 
    void __fastcall FSetSY( dword pArrayIndex, D3DVALUE pSY ); 
    D3DVALUE __fastcall FGetSZ( dword pArrayIndex ); 
    void __fastcall FSetSZ( dword pArrayIndex, D3DVALUE pSZ ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 
    dword __fastcall FGetSizeAll(); 
    D3DCOLOR __fastcall FGetSpecular( dword pArrayIndex ); 
    void __fastcall FSetSpecular( dword pArrayIndex, D3DCOLOR pSpecular ); 
    D3DVALUE __fastcall FGetTU( dword pArrayIndex ); 
    void __fastcall FSetTU( dword pArrayIndex, D3DVALUE pTU ); 
    D3DVALUE __fastcall FGetTV( dword pArrayIndex ); 
    void __fastcall FSetTV( dword pArrayIndex, D3DVALUE pTV ); 
    D3DCOLOR __fastcall FGetcolor( dword pArrayIndex ); 
    void __fastcall FSetcolor( dword pArrayIndex, D3DCOLOR pcolor ); 
    D3DVALUE __fastcall FGetrhw( dword pArrayIndex ); 
    void __fastcall FSetrhw( dword pArrayIndex, D3DVALUE prhw ); 
    D3DCOLOR __fastcall FGetspecular( dword pArrayIndex ); 
    void __fastcall FSetspecular( dword pArrayIndex, D3DCOLOR pspecular ); 
    D3DVALUE __fastcall FGetsx( dword pArrayIndex ); 
    void __fastcall FSetsx( dword pArrayIndex, D3DVALUE psx ); 
    D3DVALUE __fastcall FGetsy( dword pArrayIndex ); 
    void __fastcall FSetsy( dword pArrayIndex, D3DVALUE psy ); 
    D3DVALUE __fastcall FGetsz( dword pArrayIndex ); 
    void __fastcall FSetsz( dword pArrayIndex, D3DVALUE psz ); 
    D3DVALUE __fastcall FGettu( dword pArrayIndex ); 
    void __fastcall FSettu( dword pArrayIndex, D3DVALUE ptu ); 
    D3DVALUE __fastcall FGettv( dword pArrayIndex ); 
    void __fastcall FSettv( dword pArrayIndex, D3DVALUE ptv ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    D3DTLVERTEX* __fastcall FGetInternal_D3DTLVERTEX_Ptr(); 
    void __fastcall FSetInternal_D3DTLVERTEX( dword pArrayIndex, D3DTLVERTEX* 
    pD3DTLVERTEX ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    dword fArraySize; 
    HRESULT fErrorValue; 
    dword fSize; 
    dword fSizeAll; 

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    D3DTLVERTEX* fD3DTLVERTEX; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
