#ifndef TDDGammaRampH
#define TDDGammaRampH

// ==========================================================================
// File:         TDDGammaRamp.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons)
// Description:  This file defines the TDDGammaRamp Component 
//
// "TDx_Draw3D_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Draw3D_Library_Defns.H"
#include "TDx_Draw3D_Library_Functions.H"

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDDGammaRamp
// Description: The TDDGammaRamp component wraps the DirectX DDGAMMARAMP 
//              structure, which is used to hold the gamma ramp settings for 
//              the red, green, and blue color components of a primary 
//              surface. 
//              
//              The color value for each color component of a pixel is used 
//              to index into the corresponding TDDGammaRamp::Red, 
//              TDDGammaRamp::Green and TDDGammaRamp::Blue arrays. 
//              
//              The value referenced within the array is used to replace the 
//              existing value for that color component. 
//              For example, the whole primary surface could be made to 
//              appear bluer by increasing the values contained in 
//              TDDGammaRamp::Blue. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDDGammaRamp : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDDGammaRamp : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDDGammaRamp component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Blue
    // Description: The Blue property references an array holding the gamma 
    //              ramp for the blue color component. 
    // ----------------------------------------------------------------------
    __property word Blue[dword pIndex] = { read=FGetBlue, write=FSetBlue }; 

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDDGammaRamp method or 
    //              fget/fset. eg. D3D_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=D3D_OK }; 

    // ----------------------------------------------------------------------
    // Property:    Green
    // Description: The Green property references an array holding the gamma 
    //              ramp for the green color component. 
    // ----------------------------------------------------------------------
    __property word Green[dword pIndex] = { read=FGetGreen, write=FSetGreen 
    }; 

    // ----------------------------------------------------------------------
    // Property:    Red
    // Description: The Red property references an array holding the gamma 
    //              ramp for the red color component. 
    // ----------------------------------------------------------------------
    __property word Red[dword pIndex] = { read=FGetRed, write=FSetRed }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property contains the size in bytes of the 
    //              internal DDGAMMARAMP structure. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, nodefault }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TDDGammaRamp's internal DDGAMMARAMP structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear(); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDDGammaRamp(TComponent* Owner); 
    virtual __fastcall ~TDDGammaRamp(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property DDGAMMARAMP* Internal_DDGAMMARAMP_Ptr = { 
    read=FGetInternal_DDGAMMARAMP_Ptr, nodefault }; 

    void __fastcall Internal_DDGAMMARAMP_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    word __fastcall FGetBlue( dword pIndex ); 
    void __fastcall FSetBlue( dword pIndex, word pBlue ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    word __fastcall FGetGreen( dword pIndex ); 
    void __fastcall FSetGreen( dword pIndex, word pGreen ); 
    word __fastcall FGetRed( dword pIndex ); 
    void __fastcall FSetRed( dword pIndex, word pRed ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    DDGAMMARAMP* __fastcall FGetInternal_DDGAMMARAMP_Ptr(); 
    void __fastcall FSetInternal_DDGAMMARAMP( DDGAMMARAMP* pDDGAMMARAMP ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    HRESULT fErrorValue; 
    dword fSize; 

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    DDGAMMARAMP fDDGAMMARAMP; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
