#ifndef TDDVideoPortBandwidthH
#define TDDVideoPortBandwidthH

// ==========================================================================
// File:         TDDVideoPortBandwidth.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons)
// Description:  This file defines the TDDVideoPortBandwidth Component 
//
// "TDx_Draw3D_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Draw3D_Library_Defns.H"
#include "TDx_Draw3D_Library_Functions.H"

// ==========================================================================
// Set:         TDDVideoPortBandwidthCaps_Set
// Description: A set of mangled DirectX flag names that are used to set the 
//              TDDVideoPortBandwidth::Caps property using the BCB Object 
//              Inspector. For detailed descriptions of each flag, see the 
//              Caps property description below. 
// ==========================================================================
enum TDDVideoPortBandwidth_Caps_Flags
    {
    ddvpbcaps_destination,
    ddvpbcaps_source
    };
typedef Set< TDDVideoPortBandwidth_Caps_Flags, 
             ddvpbcaps_destination, 
             ddvpbcaps_source >  TDDVideoPortBandwidth_Caps_Set;

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDDVideoPortBandwidth
// Description: The TDDVideoPortBandwidth component wraps the DirectX 
//              DDVIDEOPORTBANDWIDTH structure, which describes bandwidth 
//              characteristics for an overlay surface which uses a specific 
//              pixel format and video port. 
//              
//              It is used by TDx_DrawVideoPort::GetBandwidthInfo(). 
//              
//              Interpretation of the various overlay size properties depend 
//              on TDDVideoPortBandwidth::Flags settings. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDDVideoPortBandwidth : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDDVideoPortBandwidth : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Caps_
    // Description: The Caps property defines flags indicating how the other 
    //              properties of this component are to be interpreted. 
    //              The described effect applies when the flag is set. 
    // Note:        This property is used to set the various DirectX Flags 
    //              for the 'Caps' property via the BCB Object Inspector. 
    //               For detailed descriptions of each flag, see the Caps 
    //              property description below. 
    // ----------------------------------------------------------------------
    __property TDDVideoPortBandwidth_Caps_Set Caps_ = { read=FGetCaps_, 
    write=FSetCaps_, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    Colorkey
    // Description: The Colorkey property defines a stretch factor indicating 
    //              the size requirements for the video port to display an 
    //              overlay with color keying, relative to the dimensions 
    //              provided in the TDx_DrawVideoPort::GetBandwidthInfo call. 
    // ----------------------------------------------------------------------
    __property dword Colorkey = { read=FGetColorkey, write=FSetColorkey, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Overlay
    // Description: The Overlay property defines a stretch factor indicating 
    //              the size requirements for the video port to display an 
    //              overlay, relative to the dimensions provided in the 
    //              TDx_DrawVideoPort::GetBandwidthInfo() call. 
    // ----------------------------------------------------------------------
    __property dword Overlay = { read=FGetOverlay, write=FSetOverlay, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property defines the size, in bytes, of the 
    //              internal structure used by this component. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, 
    default=sizeof(DDVIDEOPORTBANDWIDTH) }; 

    // ----------------------------------------------------------------------
    // Property:    YInterpAndColorkey
    // Description: The YInterpAndColorkey property defines a stretch factor 
    //              indicating the size requirements for the display of an 
    //              overlay with Y axis interpolation and color keying, 
    //              relative to the dimensions provided in the 
    //              TDx_DrawVideoPort::GetBandwidthInfo() call. 
    // ----------------------------------------------------------------------
    __property dword YInterpAndColorkey = { read=FGetYInterpAndColorkey, 
    write=FSetYInterpAndColorkey, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    YInterpolate
    // Description: The YInterpolate property defines a stretch factor 
    //              indicating the size requirements for the display of an 
    //              overlay with Y axis interpolation, relative to the 
    //              dimensions provided in the 
    //              TDx_DrawVideoPort::GetBandwidthInfo call. 
    // ----------------------------------------------------------------------
    __property dword YInterpolate = { read=FGetYInterpolate, 
    write=FSetYInterpolate, default=0 }; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDDVideoPortBandwidth component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Caps
    // Description: The Caps property defines flags indicating how the other 
    //              properties of this component are to be interpreted. 
    //              The described effect applies when the flag is set. 
    // Flags:       DDVPBCAPS_DESTINATION -
    //                The device capabilities are described in terms of the 
    //                overlay's minimum stretch factor. 
    //                This means bandwidth information provided for this 
    //                device refers to the destination overlay size. 
    //                The stretch factors detailed in the other properties of 
    //                this component represent the minimum stretch factor 
    //                that needs to be applied to successfully display an 
    //                overlay of the size specified in the 
    //                TDx_DrawVideoPort::GetBandwidthInfo() call. 
    //              DDVPBCAPS_SOURCE -
    //                The device capabilities are described in terms of the 
    //                required source overlay size. 
    //                This means bandwidth information provided for this 
    //                device refers to the source overlay size. 
    //                The stretch factors detailed in the other properties of 
    //                this component represent how much the source overlay 
    //                must be shrunk to successfully display an overlay of 
    //                the size specified in the 
    //                TDx_DrawVideoPort::GetBandwidthInfo() call. 
    //                A value of 1000 indicates no modifications are 
    //                required, a smaller value indicates the suggested 
    //                source rectangle was too large. 
    // ----------------------------------------------------------------------
    __property dword Caps = { read=FGetCaps, write=FSetCaps, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDDVideoPortBandwidth method or 
    //              fget/fset. eg. D3D_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=D3D_OK }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TDDVideoPortBandwidth's internal DDVIDEOPORTBANDWIDTH 
    //              structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear(); 

    // ----------------------------------------------------------------------
    // Property:    Caps_Strings()
    // Description: This property returns a TStringList containing current 
    //              flag values for the Caps property. Note: Any memory 
    //              associated with these strings is released on destruction. 
    // ----------------------------------------------------------------------
    __property TStringList* Caps_Strings = { read=FGetCaps_Strings };

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDDVideoPortBandwidth(TComponent* Owner); 
    virtual __fastcall ~TDDVideoPortBandwidth(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property DDVIDEOPORTBANDWIDTH* Internal_DDVIDEOPORTBANDWIDTH_Ptr = { 
    read=FGetInternal_DDVIDEOPORTBANDWIDTH_Ptr, nodefault }; 

    void __fastcall Internal_DDVIDEOPORTBANDWIDTH_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Object Inspector 'Flag' Access Methods
    // ----------------------------------------------------------------------
    TDDVideoPortBandwidth_Caps_Set __fastcall FGetCaps_(); 
    void __fastcall FSetCaps_( TDDVideoPortBandwidth_Caps_Set pCaps_ ); 

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    dword __fastcall FGetCaps(); 
    void __fastcall FSetCaps( dword pCaps ); 
    dword __fastcall FGetColorkey(); 
    void __fastcall FSetColorkey( dword pColorkey ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    dword __fastcall FGetOverlay(); 
    void __fastcall FSetOverlay( dword pOverlay ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 
    dword __fastcall FGetYInterpAndColorkey(); 
    void __fastcall FSetYInterpAndColorkey( dword pYInterpAndColorkey ); 
    dword __fastcall FGetYInterpolate(); 
    void __fastcall FSetYInterpolate( dword pYInterpolate ); 

    // ----------------------------------------------------------------------
    // Flag_Strings Access Methods
    // ----------------------------------------------------------------------
    TStringList* __fastcall FGetCaps_Strings();

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    DDVIDEOPORTBANDWIDTH* __fastcall FGetInternal_DDVIDEOPORTBANDWIDTH_Ptr(); 
    void __fastcall FSetInternal_DDVIDEOPORTBANDWIDTH( DDVIDEOPORTBANDWIDTH* 
    pDDVIDEOPORTBANDWIDTH ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    HRESULT fErrorValue; 

    // ----------------------------------------------------------------------
    // Flags_Strings Variables
    // ----------------------------------------------------------------------
    TStringList* fCaps_Strings;

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    DDVIDEOPORTBANDWIDTH fDDVIDEOPORTBANDWIDTH; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
