#ifndef TDDVideoPortInfoH
#define TDDVideoPortInfoH

// ==========================================================================
// File:         TDDVideoPortInfo.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons)
// Description:  This file defines the TDDVideoPortInfo Component 
//
// "TDx_Draw3D_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Draw3D_Library_Defns.H"
#include "TDx_Draw3D_Library_Functions.H"
// --------------------------------------------------------------------------
// external classes used by TDDVideoPortInfo properties.
#include "TDDPixelFormat.H"

// ==========================================================================
// Set:         TDDVideoPortInfoVPFlags_Set
// Description: A set of mangled DirectX flag names that are used to set the 
//              TDDVideoPortInfo::VPFlags property using the BCB Object 
//              Inspector. For detailed descriptions of each flag, see the 
//              VPFlags property description below. 
// ==========================================================================
enum TDDVideoPortInfo_VPFlags_Flags
    {
    ddvp_autoflip,
    ddvp_convert,
    ddvp_crop,
    ddvp_ignorevbixcrop,
    ddvp_interleave,
    ddvp_mirrorleftright,
    ddvp_mirrorupdown,
    ddvp_overridebobweave,
    ddvp_prescale,
    ddvp_skipevenfields,
    ddvp_skipoddfields,
    ddvp_syncmaster,
    ddvp_vbiconvert,
    ddvp_vbinointerleave,
    ddvp_vbinoscale
    };
typedef Set< TDDVideoPortInfo_VPFlags_Flags, 
             ddvp_autoflip, 
             ddvp_vbinoscale >  TDDVideoPortInfo_VPFlags_Set;

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDDVideoPortInfo
// Description: The TDDVideoPortInfo component wraps the DirectX 
//              DDVIDEOPORTINFO structure, which describes the transfer of 
//              video data from a video port to a surface. 
//              
//              It is used by TDx_DrawVideoPort::UpdateVideo() and 
//              TDx_DrawVideoPort::StartVideo. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDDVideoPortInfo : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDDVideoPortInfo : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    VPFlags_
    // Description: The VPFlags property defines flags indicating what video 
    //              port options are to be utilized when transferring data to 
    //              a surface. 
    //              The described effect applies when the flag is set. 
    // Note:        This property is used to set the various DirectX Flags 
    //              for the 'VPFlags' property via the BCB Object Inspector. 
    //               For detailed descriptions of each flag, see the VPFlags 
    //              property description below. 
    // ----------------------------------------------------------------------
    __property TDDVideoPortInfo_VPFlags_Set VPFlags_ = { read=FGetVPFlags_, 
    write=FSetVPFlags_, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    InputFormat
    // Description: The InputFormat property references a TDDPixelFormat 
    //              component describing the pixel format that will be 
    //              written to the video port. 
    //              This is usually identical to the surface's pixel format 
    //              but may differ if the video port is set perform 
    //              conversion by setting the corresponding 
    //              TDDVideoPortInfo::VPFlags values. 
    // ----------------------------------------------------------------------
    __property TDDPixelFormat* InputFormat = { read=FGetInputFormat, 
    write=FSetInputFormat, default=NULL }; 

    // ----------------------------------------------------------------------
    // Property:    OriginX
    // Description: The OriginX property defines the X coordinate indicating 
    //              where to start feeding video data to a surface. 
    // ----------------------------------------------------------------------
    __property dword OriginX = { read=FGetOriginX, write=FSetOriginX, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    OriginY
    // Description: The OriginY property defines the Y coordinate indicating 
    //              where to start feeding video data to a surface. 
    // ----------------------------------------------------------------------
    __property dword OriginY = { read=FGetOriginY, write=FSetOriginY, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    PrescaleHeight
    // Description: The PrescaleHeight property defines how much pre-scaling 
    //              or zooming, along the Y axis, is to be done before 
    //              writing the video data to a surface. 
    //              This property is optional. 
    // ----------------------------------------------------------------------
    __property dword PrescaleHeight = { read=FGetPrescaleHeight, 
    write=FSetPrescaleHeight, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    PrescaleWidth
    // Description: The PrescaleWidth property defines how much pre-scaling 
    //              or zooming, along the X axis, is to be done before 
    //              writing the video data to a surface. 
    //              This property is optional. 
    // ----------------------------------------------------------------------
    __property dword PrescaleWidth = { read=FGetPrescaleWidth, 
    write=FSetPrescaleWidth, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property defines the size, in bytes, of the 
    //              internal structure used by this component. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, 
    default=sizeof(DDVIDEOPORTINFO) }; 

    // ----------------------------------------------------------------------
    // Property:    VBIHeight
    // Description: The VBIHeight property defines the amount of data, in 
    //              scan lines, within the VBI. 
    // ----------------------------------------------------------------------
    __property dword VBIHeight = { read=FGetVBIHeight, write=FSetVBIHeight, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    VBIInputFormat
    // Description: The VBIInputFormat property references a TDDPixelFormat 
    //              component describing the pixel format that will be 
    //              written to a surface. 
    //              This is usually identical to the input pixel format but 
    //              may differ if the video port is set perform conversion by 
    //              setting the corresponding TDDVideoPortInfo::VPFlags 
    //              values. 
    // ----------------------------------------------------------------------
    __property TDDPixelFormat* VBIInputFormat = { read=FGetVBIInputFormat, 
    write=FSetVBIInputFormat, default=NULL }; 

    // ----------------------------------------------------------------------
    // Property:    VBIOutputFormat
    // Description: The VBIOutputFormat property references a TDDPixelFormat 
    //              component describing the output pixel format of the data 
    //              contained within the VBI of the video stream. 
    // ----------------------------------------------------------------------
    __property TDDPixelFormat* VBIOutputFormat = { read=FGetVBIOutputFormat, 
    write=FSetVBIOutputFormat, default=NULL }; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDDVideoPortInfo component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Crop
    // Description: The Crop property references a TRect defining the 
    //              rectangle used for cropping video data. 
    //              This property is optional. 
    // ----------------------------------------------------------------------
    __property TRect Crop = { read=FGetCrop, write=FSetCrop, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDDVideoPortInfo method or 
    //              fget/fset. eg. D3D_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=D3D_OK }; 

    // ----------------------------------------------------------------------
    // Property:    VPFlags
    // Description: The VPFlags property defines flags indicating what video 
    //              port options are to be utilized when transferring data to 
    //              a surface. 
    //              The described effect applies when the flag is set. 
    // Flags:       DDVP_AUTOFLIP -
    //                Autoflip to the next destination overlay surface in the 
    //                flip chain as each VREF occurs. 
    //                Flips will occur every VREF unless the fields are being 
    //                interleaved in memory, in which case flips will occur 
    //                every second VREF. 
    //              DDVP_CONVERT -
    //                Video data is to be color converted before it is 
    //                written to a surface by using the pixel format 
    //                information referenced by 
    //                TDDVideoPortInfo::OutputFormat. 
    //              DDVP_CROP -
    //                Video data is to be cropped using the rectangle 
    //                referenced by TDDVideoPortInfo::Crop, before writing it 
    //                to a surface 
    //              DDVP_IGNOREVBIXCROP -
    //                X axis cropping coordinates are to be ignored when 
    //                cropping oversampled VBI data. 
    //              DDVP_INTERLEAVE -
    //                Interlaced fields are to be interleaved in memory. 
    //              DDVP_MIRRORLEFTRIGHT -
    //                Image data is to be irrorred from left to right when 
    //                writing to the frame buffer. 
    //              DDVP_MIRRORUPDOWN -
    //                Image data is to be mirrorred from top to bottom when 
    //                writing it to the frame buffer. 
    //              DDVP_OVERRIDEBOBWEAVE -
    //                Override any automatic display methods chosen by the 
    //                driver and use only the display method set by the 
    //                caller when they created the overlay surface. 
    //              DDVP_PRESCALE -
    //                Pre-scale or zoom the video data based on the values in 
    //                TDDVideoPortInfo::PrescaleHeight and 
    //                TDDVideoPortInfo::PrescaleWidth. 
    //              DDVP_SKIPEVENFIELDS -
    //                Ignore the even field of the video frames. 
    //              DDVP_SKIPODDFIELDS -
    //                Ignore the odd field of the video frames. 
    //              DDVP_SYNCMASTER -
    //                Lock the graphics refresh rate to the video port by 
    //                using the video port VREF as the graphics VREF. 
    //              DDVP_VBICONVERT -
    //                Convert VBI data contained in the video stream using 
    //                the pixel format information contained in 
    //                TDDVideoPortInfo::VBIOutputFormat. 
    //              DDVP_VBINOINTERLEAVE -
    //                Disable interleaving in memory for data contained 
    //                within the VBI. 
    //              DDVP_VBINOSCALE -
    //                Do not perform any scaling on VBI data contained within 
    //                the video stream. 
    // ----------------------------------------------------------------------
    __property dword VPFlags = { read=FGetVPFlags, write=FSetVPFlags, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TDDVideoPortInfo's internal DDVIDEOPORTINFO 
    //              structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear(); 

    // ----------------------------------------------------------------------
    // Property:    VPFlags_Strings()
    // Description: This property returns a TStringList containing current 
    //              flag values for the VPFlags property. Note: Any memory 
    //              associated with these strings is released on destruction. 
    // ----------------------------------------------------------------------
    __property TStringList* VPFlags_Strings = { read=FGetVPFlags_Strings };

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDDVideoPortInfo(TComponent* Owner); 
    virtual __fastcall ~TDDVideoPortInfo(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property DDVIDEOPORTINFO* Internal_DDVIDEOPORTINFO_Ptr = { 
    read=FGetInternal_DDVIDEOPORTINFO_Ptr, nodefault }; 

    void __fastcall Internal_DDVIDEOPORTINFO_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Object Inspector 'Flag' Access Methods
    // ----------------------------------------------------------------------
    TDDVideoPortInfo_VPFlags_Set __fastcall FGetVPFlags_(); 
    void __fastcall FSetVPFlags_( TDDVideoPortInfo_VPFlags_Set pVPFlags_ ); 

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    TRect __fastcall FGetCrop(); 
    void __fastcall FSetCrop( TRect pCrop ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    TDDPixelFormat* __fastcall FGetInputFormat(); 
    void __fastcall FSetInputFormat( TDDPixelFormat* pInputFormat ); 
    dword __fastcall FGetOriginX(); 
    void __fastcall FSetOriginX( dword pOriginX ); 
    dword __fastcall FGetOriginY(); 
    void __fastcall FSetOriginY( dword pOriginY ); 
    dword __fastcall FGetPrescaleHeight(); 
    void __fastcall FSetPrescaleHeight( dword pPrescaleHeight ); 
    dword __fastcall FGetPrescaleWidth(); 
    void __fastcall FSetPrescaleWidth( dword pPrescaleWidth ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 
    dword __fastcall FGetVBIHeight(); 
    void __fastcall FSetVBIHeight( dword pVBIHeight ); 
    TDDPixelFormat* __fastcall FGetVBIInputFormat(); 
    void __fastcall FSetVBIInputFormat( TDDPixelFormat* pVBIInputFormat ); 
    TDDPixelFormat* __fastcall FGetVBIOutputFormat(); 
    void __fastcall FSetVBIOutputFormat( TDDPixelFormat* pVBIOutputFormat ); 
    dword __fastcall FGetVPFlags(); 
    void __fastcall FSetVPFlags( dword pVPFlags ); 

    // ----------------------------------------------------------------------
    // Flag_Strings Access Methods
    // ----------------------------------------------------------------------
    TStringList* __fastcall FGetVPFlags_Strings();

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    DDVIDEOPORTINFO* __fastcall FGetInternal_DDVIDEOPORTINFO_Ptr(); 
    void __fastcall FSetInternal_DDVIDEOPORTINFO( DDVIDEOPORTINFO* 
    pDDVIDEOPORTINFO ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    HRESULT fErrorValue; 
    TDDPixelFormat* fInputFormat; 
    TDDPixelFormat* fVBIInputFormat; 
    TDDPixelFormat* fVBIOutputFormat; 

    // ----------------------------------------------------------------------
    // Flags_Strings Variables
    // ----------------------------------------------------------------------
    TStringList* fVPFlags_Strings;

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    DDVIDEOPORTINFO fDDVIDEOPORTINFO; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
