#ifndef TDx_DrawClipperH
#define TDx_DrawClipperH

// ==========================================================================
// File:         TDx_DrawClipper.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons)
// Description:  This file defines the TDx_DrawClipper Component 
//
// "TDx_Draw3D_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Draw3D_Library_Defns.H"
#include "TDx_Draw3D_Library_Functions.H"
// --------------------------------------------------------------------------
// external classes used by TDx_DrawClipper methods.
class TDx_Draw;

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDx_DrawClipper
// Description: The TDx_DrawClipper component wraps the DirectX 
//              IDirectDrawClipper interface, which is used to manage clip 
//              lists. 
//              
//              It will set and retrieve the window handle used to obtain 
//              clipping information, initialize a manually created 
//              TDx_DrawClipper component, set and retrieve the clip list and 
//              also determine if the clip list has been changed. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDx_DrawClipper : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDx_DrawClipper : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Event:       OnCreate()
    // Description: The OnCreate() event is triggered by the 
    //              TDx_DrawClipper::Create() method after it has 
    //              successfully created the internal LPDIRECTDRAWCLIPPER 
    //              used within the TDx_DrawClipper component. 
    //              
    //              Note that since this event is only called upon successful 
    //              creation, this event is an ideal place for code that is 
    //              to be activated only once, immediately upon the 
    //              successful creation of the LPDIRECTDRAWCLIPPER interface. 
    //              
    //              For example, you could retrieve the TDx_DrawClipper 
    //              component's capabilities, Create() dependant components, 
    //              allocate memory, etc. 
    //              
    //              When used in conjunction with the 
    //              TDx_DrawClipper::OnDestroy() event, you can fully 
    //              automate and insulate your code on a per-component basis. 
    // ----------------------------------------------------------------------
    __property TDx_Event OnCreate = {read=FOnCreate, write=FOnCreate, 
    nodefault}; 

    // ----------------------------------------------------------------------
    // Event:       OnDestroy()
    // Description: The OnDestroy() event is triggered by the 
    //              TDx_DrawClipper::Destroy() method, just before the 
    //              TDx_DrawClipper internal LPDIRECTDRAWCLIPPER interface is 
    //              destroyed. 
    //              
    //              Note: This event is an ideal place to place code that 
    //              reverses the results of the TDx_DrawClipper::OnCreate() 
    //              event. For example, assume you have already setup the 
    //              TDx_DrawClipper::OnCreate() event to retrieve the 
    //              component's capabilities, Create() a dependant component 
    //              and allocate some memory; 
    //              You can then fully automate and insulate your code on a 
    //              per-component basis by placing code in the 
    //              TDx_DrawClipper::OnDestroy() event to deallocate memory, 
    //              Destroy() the dependant components and, when required, 
    //              destroy the capabilities component. 
    // ----------------------------------------------------------------------
    __property TDx_Event OnDestroy = {read=FOnDestroy, write=FOnDestroy, 
    nodefault}; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDx_DrawClipper component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Created
    // Description: The Created property is true if the internal 
    //              LPDIRECTDRAWCLIPPER used in this component has been 
    //              successfully created, otherwise Created is false. 
    //              
    //              To create the internal LPDIRECTDRAWCLIPPER, call the 
    //              TDx_DrawClipper::Create() method. 
    // ----------------------------------------------------------------------
    __property bool Created = { read=FGetCreated, write=FSetCreated, 
    default=false }; 

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDx_DrawClipper method or 
    //              fget/fset. eg. D3D_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=D3D_OK }; 

    // ----------------------------------------------------------------------
    // Method:      Create()
    // Description: The Create() method is used to automatically create the 
    //              internal LPDIRECTDRAWCLIPPER interface used in the 
    //              TDx_DrawClipper component and must be called before any 
    //              methods of the TDx_DrawClipper component will function. 
    // Params:      pDraw - 
    //                A TDx_Draw object that will manage the clipper. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall Create( TDx_Draw* pDraw ); 

    // ----------------------------------------------------------------------
    // Method:      Destroy()
    // Description: The Destroy() method is used to automatically destroy the 
    //              internal LPDIRECTDRAWCLIPPER interface used in the 
    //              TDx_DrawClipper component and should be called when the 
    //              internal interface is no longer required. 
    //              
    //              Note: This method is called by the component's 
    //              destructor. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall Destroy(); 

    // ----------------------------------------------------------------------
    // Method:      GetClipList()
    // Description: The TDx_DrawClipper::GetClipList method will retrieve a 
    //              copy of this clipper's clip list. 
    //              
    //              Subsets of a clip list can be retrieved using the Rect 
    //              parameter of this method to pass a rectangle which clips 
    //              the clip list. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              DDERR_GENERIC 
    //              DDERR_INVALIDCLIPLIST 
    //              DDERR_INVALIDOBJECT 
    //              DDERR_INVALIDPARAMS 
    //              DDERR_NOCLIPLIST 
    //              DDERR_REGIONTOOSMALL 
    // Params:      pRect - 
    //                The Rect parameter references a TRect for clipping the 
    //                clip list. 
    //                Set this parameter to NULL to retrieve the whole clip 
    //                list. 
    //              pClipList - 
    //                The ClipList parameter references a RGNDATA structure for 
    //                holding the clip list if this method returns 
    //                successfully. 
    //                When this parameter is set to NULL the Size parameter of 
    //                this method will be filled with the size, in bytes, of 
    //                the entire clip list. 
    //              pSize - 
    //                The Size parameter defines the size, in bytes, of the 
    //                buffer referenced by the ClipList parameter of this 
    //                method. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall GetClipList( TRect* pRect, RGNDATA* pClipList, 
    dword* pSize ); 

    // ----------------------------------------------------------------------
    // Method:      GetHWnd()
    // Description: The TDx_DrawClipper::GetHWnd method will return the 
    //              window handle associated with this clipper by 
    //              TDx_DrawClipper::SetHWnd(). 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              DDERR_INVALIDOBJECT 
    //              DDERR_INVALIDPARAMS 
    // Params:      pWnd - 
    //                The Wnd parameter will reference the clipper's previously 
    //                associated window handle if this method returns 
    //                successfully. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall GetHWnd( HWND* pWnd ); 

    // ----------------------------------------------------------------------
    // Method:      IsClipListChanged()
    // Description: The TDx_DrawClipper::IsClipListChanged method will 
    //              indicate whether the clip list has been changed. 
    //              
    //              This method is only useful when the clipper is associated 
    //              with a window handle. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              DDERR_INVALIDOBJECT 
    //              DDERR_INVALIDPARAMS 
    // Params:      pChanged - 
    //                The Changed parameter will reference the current state of 
    //                the clip list if this method returns successfully. 
    //                TRUE if the clip list has changed. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall IsClipListChanged( bool* pChanged ); 

    // ----------------------------------------------------------------------
    // Method:      SetClipList()
    // Description: The TDx_DrawClipper::SetClipList method will set or 
    //              delete the clip lists used by TDx_DrawSurface::Blt() and 
    //              TDx_DrawSurface::UpdateOverlay for surfaces to which the 
    //              clipper is attached. 
    //              
    //              A clip list cannot be set when there is a window handle 
    //              associated with the clipper. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              DDERR_CLIPPERISUSINGHWND 
    //              DDERR_INVALIDCLIPLIST 
    //              DDERR_INVALIDOBJECT 
    //              DDERR_INVALIDPARAMS 
    //              DDERR_OUTOFMEMORY 
    // Params:      pClipList - 
    //                The ClipList parameter references a RGNDATA structure 
    //                holding the clip list that is to be set. 
    //                Setting this property to NULL when a clip list already 
    //                exists will delete that clip list. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall SetClipList( RGNDATA* pClipList ); 

    // ----------------------------------------------------------------------
    // Method:      SetHWnd()
    // Description: The TDx_DrawClipper::SetHWnd method will set the window 
    //              handle used to obtain the clipping information. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              DDERR_INVALIDCLIPLIST 
    //              DDERR_INVALIDOBJECT 
    //              DDERR_INVALIDPARAMS 
    //              DDERR_OUTOFMEMORY 
    // Params:      pWnd - 
    //                The Wnd parameter defines the window handle to be used 
    //                for obtaining clip information. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall SetHWnd( HWND pWnd ); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDx_DrawClipper(TComponent* Owner); 
    virtual __fastcall ~TDx_DrawClipper(); 

    // ----------------------------------------------------------------------
    // The following properties and methods are used internally by 
    // TDx_Draw3D_Library and should not be used. 
    // ----------------------------------------------------------------------
    __property LPDIRECTDRAWCLIPPER Internal_LPDIRECTDRAWCLIPPER = { 
    read=FGetInternal_LPDIRECTDRAWCLIPPER, 
    write=FSetInternal_LPDIRECTDRAWCLIPPER, nodefault }; 
    __property LPDIRECTDRAWCLIPPER* Internal_LPDIRECTDRAWCLIPPER_Ptr = { 
    read=FGetInternal_LPDIRECTDRAWCLIPPER_Ptr, nodefault }; 
    void __fastcall Internal_LPDIRECTDRAWCLIPPER_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    bool __fastcall FGetCreated(); 
    void __fastcall FSetCreated( bool pCreated ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Interface Access
    // ----------------------------------------------------------------------
    LPDIRECTDRAWCLIPPER __fastcall FGetInternal_LPDIRECTDRAWCLIPPER(); 
    void __fastcall FSetInternal_LPDIRECTDRAWCLIPPER( LPDIRECTDRAWCLIPPER 
    pLPDIRECTDRAWCLIPPER ); 
    LPDIRECTDRAWCLIPPER* __fastcall FGetInternal_LPDIRECTDRAWCLIPPER_Ptr(); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    bool fCreated; 
    HRESULT fErrorValue; 

    // ----------------------------------------------------------------------
    // Interface Variables 
    // ----------------------------------------------------------------------
    LPDIRECTDRAWCLIPPER fLPDIRECTDRAWCLIPPER; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Event FOnCreate;
    TDx_Event FOnDestroy;
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
